/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package simple.documentation.framework.callouts;

import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.text.BadLocationException;

import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.callouts.AuthorCalloutRenderingInformation;
import ro.sync.ecss.extensions.api.callouts.CalloutsRenderingInformationProvider;
import ro.sync.ecss.extensions.api.highlights.AuthorPersistentHighlight;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.exml.view.graphics.Color;


/**
 * Callouts rendering provider.
 * 
 * @author marius
 */
public class CalloutsRenderingProvider extends CalloutsRenderingInformationProvider {
  /**
   * The author access.
   */
  private final AuthorAccess authorAccess;

  /**
   * Constructor.
   * 
   * @param authorAccess The author access.
   */
  public CalloutsRenderingProvider(AuthorAccess authorAccess) {
    this.authorAccess = authorAccess;
  }

  @Override
  public boolean shouldRenderAsCallout(AuthorPersistentHighlight persistentHighlight) {
    return true;
  }

  @Override
  public AuthorCalloutRenderingInformation getCalloutRenderingInformation(
      final AuthorPersistentHighlight hl) {
    return new AuthorCalloutRenderingInformation() {

      @Override
      public long getTimestamp() {
        return -1;
      }

      @Override
      public String getContentFromTarget(int arg0) {
        try {
          AuthorDocumentController documentController = authorAccess.getDocumentController();
          AuthorNode nodeAtOffset = documentController.getNodeAtOffset(hl.getEndOffset());
          int startOffset = hl.getStartOffset() - nodeAtOffset.getStartOffset();
          int endOffset = hl.getEndOffset() - nodeAtOffset.getEndOffset();
          String textContent = nodeAtOffset.getTextContent();
          return textContent.substring(startOffset, endOffset);
        } catch (BadLocationException e) {
          e.printStackTrace();
          return null;
        }
      }

      @Override
      public String getComment(int arg0) {
        return hl.getClonedProperties().get(AuthorPersistentHighlight.COMMENT_ATTRIBUTE);
      }

      @Override
      public Color getColor() {
        return Color.COLOR_DARK_YELLOW;
      }

      @Override
      public String getCalloutType() {
        return "Paragraph review";
      }

      @Override
      public String getAuthor() {
        return "para_reviewer";
      }

      @Override
      public Map<String, String> getAdditionalData() {
        LinkedHashMap<String,String> clonedProperties = hl.getClonedProperties();
        clonedProperties.put("Level", "Superior");
        return clonedProperties;
      }
    };
  }
}
