/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.contentcompletion.xml;

import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.annotations.obfuscate.SkipLevel;
import ro.sync.annotations.obfuscate.SkipObfuscate;
import ro.sync.exml.EXMLResourceBoundle;
import ro.sync.exml.MessageBundle;
import ro.sync.exml.Tags;
import ro.sync.ui.application.ApplicationThread;
import ro.sync.ui.html.HTMLDocumentationUtil;
import ro.sync.xml.CatalogResolverUtil;
import ro.sync.xml.XmlUtil;
import ro.sync.xml.parser.ParserCreator;

/**
 * Style guide schema manager filter base. 
 * The default implementation adds annotations to elements and attributes by looking into a mapping file URI which is passed through the XML Catalog system...
 * 
  * @since 15
 * <br>
 * <br>
 * *********************************
 * <br>
 * EXPERIMENTAL - Subject to change
 * <br>
 * ********************************
 * </br>
 * <p>Please note that this API is not marked as final and it can change in one of the next versions of the application. If you have suggestions, 
 * comments about it, please let us know.</p> 
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
@SkipObfuscate(classes = SkipLevel.NOT_SPECIFIED, fields = SkipLevel.NOT_SPECIFIED, methods = SkipLevel.PUBLIC)
public abstract class StyleGuideSchemaManagerFilterBase extends SchemaManagerFilterBase {
  
  /**
   * The messages resource bundle.
   */
  protected static final MessageBundle messages = EXMLResourceBoundle.getResourceBundleInstance();
  /**
   * The logger used for logging.
   */
  private static Logger logger = Logger.getLogger(StyleGuideSchemaManagerFilterBase.class);
  /**
   * Location of the mapping file.
   */
  private final String locationOfMappingFile;
  
  /**
   * Constructor.
   * @param locationOfMappingFile Location of the mapping file.
   */
  public StyleGuideSchemaManagerFilterBase(String locationOfMappingFile) {
    this.locationOfMappingFile = locationOfMappingFile;
  }

  /**
   * @see ro.sync.contentcompletion.xml.SchemaManagerFilter#filterElements(java.util.List, ro.sync.contentcompletion.xml.WhatElementsCanGoHereContext)
   */
  @Override
  public List<CIElement> filterElements(List<CIElement> elements,
      WhatElementsCanGoHereContext context) {
    // Avoid NPE
    if (elements != null) {
      for (CIElement ciElement : elements) {
        if (ciElement != null) {
          AnnotationContent htdoc = getHTMLDocumentation(ciElement.getName());
          if(htdoc != null) {
            try {
              ciElement.setAnnotation(mergeExistingAnnotation(ciElement.getAnnotation(), htdoc));
            } catch(Exception ex) {
              logger.error(ex, ex);
            }
          }
        }
      }
    }
    return elements;
  }
  
  /**
   * Merge the existing annotation with the annotation content.
   * @param annotation  The existing annotation.
   * @param htmlAnnotationContent The HTML annotation content
   * @return  The merged annotation.
   */
  private String mergeExistingAnnotation(String annotation, AnnotationContent htmlAnnotationContent) {
    if(htmlAnnotationContent.htmlContent != null) {
      //Use the full HTML annotation
      annotation = htmlAnnotationContent.htmlContent;
    } else {
      //Maybe we have just links.
      if(!htmlAnnotationContent.linksContent.isEmpty()) {
        StringBuilder newAnnotation = new StringBuilder();
        if (annotation != null) {
          int bodyIdx = annotation.indexOf("</body>");
          if (bodyIdx == -1) {
            //Build the new annotation.
            newAnnotation.append("<html><body>");
            //Escape for HTML rendering
            annotation = XmlUtil.escape(annotation);
            //Replace line breaks with <br>s
            annotation = annotation.replaceAll("\n", "<br />");
            newAnnotation.append("<div>").append(annotation).append("</div>\n");
          } else {
            newAnnotation.append(annotation.substring(0, bodyIdx));
          }
        } else {
          newAnnotation.append("<html><body>");
        }
        
        // Append all links to the end
        int size = htmlAnnotationContent.linksContent.size();
        for (int i = 0; i < size; i++) {
          if (newAnnotation.toString().indexOf(htmlAnnotationContent.linksContent.get(i)) == -1) {
            newAnnotation.append("<div>");
            newAnnotation.append(htmlAnnotationContent.linksContent.get(i));
            newAnnotation.append("</div>");
          }
        }
        //End HTML content.
        newAnnotation.append("</body></html>");
        annotation = newAnnotation.toString();
      }
    }
    return annotation;
  }

  /**
   * @see ro.sync.contentcompletion.xml.SchemaManagerFilter#filterAttributes(java.util.List, ro.sync.contentcompletion.xml.WhatAttributesCanGoHereContext)
   */
  @Override
  public List<CIAttribute> filterAttributes(List<CIAttribute> attributes,
      WhatAttributesCanGoHereContext context) {
    if (attributes != null) {
      for (CIAttribute ciAttribute : attributes) {
        AnnotationContent styleGuideDoc = getHTMLDocumentation("@" + ciAttribute.getName());
        if(styleGuideDoc != null) {
          ciAttribute.setAnnotation(mergeExistingAnnotation(ciAttribute.getAnnotation(), styleGuideDoc));
        }
      }
    }
    return attributes;
  }
  
  /**
   * Get element description, contributes HTML annotation to it..
   * 
   * @param element The CIElement
   * @param ctxt    The context.
   * @return        The modified CIElement with HTML annotation.
   */
  @Override
  public CIElement getElementDescription(CIElement element, Context ctxt) {
      if(element != null) {
        AnnotationContent doc = getHTMLDocumentation(element.getName());
        if(doc != null) {
          try {
            element.setAnnotation(mergeExistingAnnotation(element.getAnnotation(), doc));
          } catch(Exception ex) {
            logger.error(ex, ex);
          }
        }
      }
      return element;
    }
  
  /**
   * The annotation HTML content class.
   */
  private static class AnnotationContent {
    /**
     * The HTML content.
     */
    private String htmlContent;

    /**
     * Links to HTML files.
     */
    private final List<String> linksContent = new ArrayList<String>(1);
  }

  /**
   * Style guide element mapped to HTML content.
   */
  private final Map<String, Map<String, AnnotationContent>> STYLE_GUIDE_ELEM_TO_HTML_CONTENT = new HashMap<String, Map<String, AnnotationContent>>();
  
  /**
   * Get HTML documentation for an element.
   * 
   * @param elementName The name of the element for which to compute the annotation. 
   */
  private AnnotationContent getHTMLDocumentation(String elementName) {
    if(STYLE_GUIDE_ELEM_TO_HTML_CONTENT.get(locationOfMappingFile) == null) {
      parseStyleGuideConfigurationMap();
    }
    return STYLE_GUIDE_ELEM_TO_HTML_CONTENT.get(locationOfMappingFile).get(elementName);
  }

  /**
   * Parse the Style Guide configuration map.
   */
  private void parseStyleGuideConfigurationMap() {
    //First time, init from the XML configuration file.
    final Map<String, AnnotationContent> elemsMap = new HashMap<String, AnnotationContent>();
    STYLE_GUIDE_ELEM_TO_HTML_CONTENT.put(locationOfMappingFile, elemsMap);
    //The download thread will start to bring all the content to a map.
    ApplicationThread downloadThread = new ApplicationThread("Download style guide mapping files"){
      @Override
      protected void appRun() {
        try {
          //Find the configuration XML file.
          URL configXMLFilePath = new URL(locationOfMappingFile);
          //Pas through catalog.
          URL resolved = CatalogResolverUtil.resolvePath(null, configXMLFilePath.toString(), false, true);
          if(resolved != null) {
            configXMLFilePath = resolved;
          }
          //DOM parse.
          Document doc = ParserCreator.newDocumentBuilderFakeResolver().parse(configXMLFilePath.toString());
          if(doc != null) {
            //The XSL which will filter the HTML to obtain a more appropriate representation.
            String filterSSValue = doc.getDocumentElement().getAttribute("htmlContentFilterStylesheet");
            //URL pointing to the filter XSL
            URL filterXSLURL = null;
            if(filterSSValue != null && filterSSValue.length() > 0) {
              filterXSLURL = new URL(configXMLFilePath, filterSSValue);
            }
            NodeList mappings = doc.getElementsByTagName("mapping");
            if(mappings != null) {
              for (int i = 0; i < mappings.getLength(); i++) {
                Node mappingNode = mappings.item(i);
                if(mappingNode.getNodeType() == Node.ELEMENT_NODE){
                  Element mappingElem = (Element) mappingNode;
                  String key = null;
                  String path = null;
                  //The name of the element.
                  String elemName = mappingElem.getAttribute("elementName");
                  if(elemName != null && elemName.length() > 0) {
                    //Path to HTML file.
                    key = elemName;
                    path = mappingElem.getAttribute("path");
                  } else {
                    //Maybe maps documentation for an attribute.
                    String attributeName = mappingElem.getAttribute("attributeName");
                    if(attributeName != null && attributeName.length() > 0){
                      //Path to HTML file.
                      path = mappingElem.getAttribute("path");
                      key = "@" + attributeName;
                    }
                  }
                  //Put the HTML content in the mapping.
                  if(path != null && path.length() > 0) {
                    AnnotationContent existingContent = elemsMap.get(key);
                    if(existingContent == null) {
                      //Create an existing annotation if it does not exist.
                      existingContent = new AnnotationContent();
                      elemsMap.put(key, existingContent);
                    }
                    if("link".equals(mappingElem.getAttribute("type"))){
                      //This is actually a link.
                      String linkText = mappingElem.getAttribute("linkText");
                      if(linkText == null || linkText.length() == 0) {
                        //Fallback
                        linkText = messages.getString(Tags.MORE_DETAILS);
                      }
                      //Pass through catalog.
                      URL htmlPathAbsoluteURL = new URL(new URL(mappingElem.getBaseURI()), path);
                      URL resolvedHTMLPath = CatalogResolverUtil.resolvePath(null, htmlPathAbsoluteURL.toString(), false, true);
                      if(resolvedHTMLPath != null) {
                        htmlPathAbsoluteURL = resolvedHTMLPath;
                      }
                      StringBuilder linkHref = new StringBuilder("<a href=\"").append(htmlPathAbsoluteURL).append("\">").append(linkText).append("</a>");
                      existingContent.linksContent.add(linkHref.toString());
                    } else {
                      String htmlContent = HTMLDocumentationUtil.getFilteredHTMLContent(new URL(mappingElem.getBaseURI()), path, filterXSLURL);
                      existingContent.htmlContent = htmlContent;
                    }
                  }
                }
              }
            }
          }
        } catch(Exception ex) {
          logger.error(ex, ex);
        }     
      }
    };
    downloadThread.setPriority(Thread.MIN_PRIORITY);
    downloadThread.start();
    //Wait a little bit, if not, let is continue.
    try {
      downloadThread.join(1500);
    } catch (InterruptedException e) {
      logger.error(e, e);
    }
  }
}
