/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.css.functions;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.annotations.obfuscate.SkipLevel;
import ro.sync.annotations.obfuscate.SkipObfuscate;
import ro.sync.ecss.css.CSS;
import ro.sync.ecss.css.RelativeLength;
import ro.sync.exml.view.graphics.Color;

/**
 * Processed arguments of the oxy_label function.
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
@SkipObfuscate(classes = SkipLevel.NOT_SPECIFIED, fields = SkipLevel.PUBLIC, methods = SkipLevel.PUBLIC)
public interface LabelCSSConstants {

  /**
   * The text that will be displayed as a label. A string value.
   */
  String TEXT_PROPERTY = "text";

  /**
   * The width of the label. An {@link RelativeLength}. 
   * 
   * <p><b>Note:</b> In case you don't have the possibility to build a {@link RelativeLength} 
   * then you can use the {@link #STYLES_PROPERTY} to give the width as a string:
   * <code>
   * * {
   *   width:100px;
   * }</code>
   * </p>
   */
  String WIDTH_PROPERTY = "width";

  /**
   * The alignment of the label. A string value: left, right or center.
   */
  String TEXT_ALIGN_PROPERTY = "text-align";
  /**
   * A foreground color for the text. A {@link Color}.
   */
  String COLOR_PROPERTY = CSS.COLOR;
  /**
   * A background color for the label. A {@link Color}.
   */
  String BACKGROUND_COLOR_PROPERTY = CSS.BACKGROUND_COLOR;
  /**
   * <p>Possibility to specify CSS rules for this label.</p> 
   * 
   * <p>Example:
   * <code>
   * * {
   *   text-align:right;
   *   color:red;
   * }
   * </code>
   * </p>
   * The selectors are ignored, all rules are considered to match.
   * <p>You can also specify as CSS something like: 
   * <code>@import 'label_styles.css';</code>
   * Relative imports will be resolved relative to {@link #BASE_SYSTEM_ID}. 
   * This approach is useful to easily reuse the same styles for more 
   * oxy_labels.</p>
   * 
   * <ul><b>The following properties are handled:</b>
   * <li>font-weight, font-size, font-style, font</li>
   * <li>text-align, text-decoration</li>
   * <li>width</li>
   * <li>color, background-color</li>
   * </ul>
   */
  String STYLES_PROPERTY = "styles";
  /**
   * Base system id to be used to resolve imports from {@link #STYLES_PROPERTY}.
   * 
   * This normally is the system ID of the CSS file in which the oxy_label was 
   * encountered. 
   */
  String BASE_SYSTEM_ID = "baseSystemId";
}