package ro.sync.ecss.extensions.api;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;

/**
 * Supported cursor types for author.
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public enum CursorType {
  /**
   * Normal cursor
   */
  CURSOR_NORMAL, 
  /**
   * Text cursor
   */
  CURSOR_TEXT, 
  /**
   * Resize East cursor.
   */
  CURSOR_E_RESIZE,
  /**
   * Resize South-East cursor.
   */
  CURSOR_SE_RESIZE,
  /**
   * Resize South cursor.
   */
  CURSOR_S_RESIZE,
  /**
   * Resize horizontal cursor.
   * 
   *  @deprecated Use {@link #CURSOR_E_RESIZE} instead.
   */
  @Deprecated
  CURSOR_RESIZE,
  /**
   * Hand cursor
   */
  CURSOR_HAND,
  /**
   * Select column cursor
   */
  CURSOR_SELECT_COLUMN,
  /**
   * Select row cursor, for tables that are oriented from left to right.
   * 
   * @deprecated You should use one of the two other constants, specifying the direction.
   */
  CURSOR_SELECT_ROW,  
  /**
   * Select row cursor, for tables that are oriented from left to right.
   */
  CURSOR_SELECT_ROW_LTR,  
  /**
   * Select row cursor, for tables that are oriented from right to left.
   */
  CURSOR_SELECT_ROW_RTL,
  /**
   * Select table cursor, for tables that are oriented from left to right.
   * @deprecated You should use one of the two other constants, specifying the direction.
   */
  CURSOR_SELECT_TABLE,
  /**
   * Select row cursor, for tables that are oriented from left to right.
   */  
  CURSOR_SELECT_TABLE_LTR,  
  /**
   * Select row cursor, for tables that are oriented from right to left.
   */  
  CURSOR_SELECT_TABLE_RTL;

  /**
   * @return <code>true</code> if this cursor is one of the resize cursors. 
   */
  public boolean isResizeCursor() {
    return 
        equals(CURSOR_E_RESIZE) 
        || equals(CURSOR_SE_RESIZE) 
        || equals(CURSOR_S_RESIZE)
        || equals(CURSOR_RESIZE);
  }
}