/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.component;

import java.io.Reader;
import java.net.URL;

import javax.swing.JComponent;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.component.listeners.AuthorComponentListener;

/**
 * Provides access to a created editor + helper views and additional panels. The editor might have multiple editor pages.
 * 
 * @since 14.2
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public interface EditorComponentProvider extends ComponentProvider {
  
  /**
   * Show the location referenced by a given URL in the editor.
   * <br/>
   * If the document pointed by this URL is different than the document currently 
   * loaded in the editor page, this URL will be used to set the content to edit, 
   * to solve relative references (eg: images) and to show the location pointed by the
   * URL reference part. 
   * <br/>
   * If the document pointed by this URL is currently loaded in the editor page,
   * only the reference part of the given URL will be used to show the corresponding location
   * in the editor.  
   * 
   * @param url The URL to show location for. 
   * @param reader The reader over the URL, can be null.
   *
   * @throws AuthorComponentException When there was a load problem (eg: IOException).
   */
  public void showLocation(URL url, Reader reader) throws AuthorComponentException;
  
  /**
   * Adds an author component listener.
   * 
   * @param listener The listener.
   */
  public void addAuthorComponentListener(AuthorComponentListener listener);
  
  /**
   * Removes an author component listener.
   * 
   * @param listener The listener.
   */
  public void removeAuthorComponentListener(AuthorComponentListener listener);
  
  /**
   * Get an additional edit helper panel.
   * 
   * It can be the outline, attributes, entities, elements or model helper component, depending on the ID.
   * 
   * @param helperID One of:
   *                 <ul>
   *                  <li>AuthorComponentProvider.ATTRIBUTES_PANEL_ID,
   *                  <li>AuthorComponentProvider.ELEMENTS_PANEL_ID, 
   *                  <li>AuthorComponentProvider.ENTITIES_PANEL_ID,
   *                  <li>AuthorComponentProvider.MODEL_PANEL_ID,
   *                  <li>AuthorComponentProvider.OUTLINER_PANEL_ID constants.
   *                 </ul>
   *
   * @return The additional component.
   */
  public JComponent getAdditionalEditHelper(int helperID);
}
