/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.table.operations;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.component.AuthorClipboardObject;
import ro.sync.ecss.component.AuthorContentMetadata;
import ro.sync.ecss.extensions.api.WidthRepresentation;

/**
 * Contains information about column specification (like column specified width).
 * It can be extended to provide specific table column properties for different 
 * types of tables or document types.
 * <br/>
 * This information is requested when a column is copied or dragged and 
 * it can be used when the column must be inserted in the document (on paste or
 * drop).
 * Please note that when a column is copied the table column specification information
 * will be copied into the clipboard (the {@link AuthorClipboardObject} contains 
 * a field of {@link TableColumnSpecificationInformation} type), 
 * so it will be serialized.  
 * <br/>
 * The column specification is send as an argument to the 
 * {@link AuthorTableOperationsHandler#handleInsertColumn(AuthorTableInsertColumnArguments)} method
 * and it can be used to keep informations like column name or column width unchchanged when 
 * the column is moved or copy-pasted. 
 * 
 * @since 14
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public class TableColumnSpecificationInformation implements AuthorContentMetadata {
  
  /**
   * The serial version UID
   */
  private static final long serialVersionUID = 5273315963174368281L;
  /**
   * The column width representation. Specifies the fixed and relative width 
   * determined from the column specification.
   */
  private WidthRepresentation widthRepresentation;
  
  /**
   * Constructor. 
   * 
   * @param widthRepresentation The column width representation that specifies 
   * the fixed and relative width determined from the column specification.
   */
  public TableColumnSpecificationInformation(WidthRepresentation widthRepresentation) {
    this.widthRepresentation = widthRepresentation;
  }
  
  /**
   * @return Returns the column width representation that specifies the fixed 
   * and relative width determined from the column specification.
   */
  public WidthRepresentation getWidthRepresentation() {
    return widthRepresentation;
  }
}
