/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.webapp;

import javax.swing.text.BadLocationException;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.component.AuthorClipboardObject;
import ro.sync.ecss.extensions.api.InvalidEditException;

/**
 * Handles schema aware actions like paste.
 * 
 * @author costi
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public interface WebappAuthorSchemaAwareActionsHandler {

  /**
   * Paste a fragment.
   * 
   * @param toPaste The fragment to paste
   * @param removeSelection Remove the selection
   * @param pasteAsXml If <code>true</code> treat the pasted text as an xml fragment. Else escape it
   * and insert it.
   * 
   * @throws InvalidEditException When paste was rejected.
   */
  void handlePaste(AuthorClipboardObject toPaste, 
      boolean removeSelection, 
      boolean pasteAsXml) throws InvalidEditException;
  
  /**
   * Copy from offset to offset inclusive.
   * 
   * @param startOffset Start offset (inclusive)
   * @param endOffset End offset (inclusive)
   * @return The doc fragment with the copy text.
   * 
   * @throws BadLocationException 
   */
  AuthorClipboardObject handleCopy(int startOffset, int endOffset) throws BadLocationException;
  
  /**
   * Inserts the given character at the current caret position. Any
   * selected content is first deleted.
   * 
   * @param ch The character to insert.
   */
  void insertCharAtCurrentOffset(char ch);
  
  
  /**
   * Deletes the content between the two given offsets using information from
   * the schema associated to the document such that document is left in a valid
   * state.
   * 
   * @param start The start offset (inclusive).
   * @param end The end offset (inclusive).
   * @param backspace Whether the deletion was triggered by a backspace.
   * 
   * @return <code>true</code> if successful.
   * 
   * @throws InvalidEditException If the edit fails. 
   */
  boolean deleteBetweenOffsets(int start, int end, boolean backspace) throws InvalidEditException;

  /**
   * Deletes the character to the right of the caret if <code>del</code> is true 
   * else delete the left character.
   * 
   * @param deleteOffset The caret offset where the deletion is performed.
   * @param del Delete or Backspace.
   * @param wordLevel <code>true</code> if the delete should work at a word level.
   * @throws InvalidEditException Thrown when delete operation was rejected. 
   */
  void deleteChar(int deleteOffset, boolean del, boolean wordLevel) throws InvalidEditException;
  
}
