/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2009 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.dita.topic.table.simpletable;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.commons.table.operations.InsertRowOperationBase;
import ro.sync.ecss.extensions.commons.table.operations.InsertTableCellsContentConstants;

/**
 * Operation used to insert a simple table row for DITA.
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class InsertRowOperation extends InsertRowOperationBase implements SimpleTableConstants, 
  InsertTableCellsContentConstants{
  
  /**
   * The fragment that must be introduced in the table cells
   */
  protected String cellContent;
  
  /**
   * Constructor.
   */
  public InsertRowOperation() {
    super(new DITASimpleTableDocumentTypeHelper());
  }
  
  /**
   * @see ro.sync.ecss.extensions.commons.table.operations.InsertRowOperationBase#doOperation(ro.sync.ecss.extensions.api.AuthorAccess, ro.sync.ecss.extensions.api.ArgumentsMap)
   */
  @Override
  public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
      throws IllegalArgumentException, AuthorOperationException {
    Object cellFragmentObj =  args.getArgumentValue(CELL_FRAGMENT_ARGUMENT_NAME);
    if (cellFragmentObj instanceof String) {
      cellContent = (String) cellFragmentObj;
    } 
    if ("".equals(cellContent)) {
      cellContent = null;
    }
    super.doOperation(authorAccess, args);
  }
    
  /**
   * @see ro.sync.ecss.extensions.commons.table.operations.InsertRowOperationBase#getCellElementName(AuthorElement, int)
   */
  @Override
  protected String getCellElementName(AuthorElement tableElement, int columnIndex) {
    String tableName = tableElement.getLocalName();
    if (ELEMENT_NAME_SIMPLETABLE.equals(tableName)) {
      // The simpletable
      return ELEMENT_NAME_ENTRY_SIMPLETABLE;
    } else if (ELEMENT_NAME_CHOICETABLE.equals(tableName)) {
      // The choicetable
      switch (columnIndex) {
        case 0 : return ELEMENT_NAME_CHOPTION_CHOICETABLE;
        case 1 : return ELEMENT_NAME_CHDESC_CHOICETABLE;
        default : return null;
      }
    } else if (ELEMENT_NAME_PROPERTIES.equals(tableName)) {
      // The properties table
      switch (columnIndex) {
        case 0 : return ELEMENT_NAME_PROPTYPE_PROPERTIES;
        case 1 : return ELEMENT_NAME_PROPVALUE_PROPERTIES;
        case 2 : return ELEMENT_NAME_PROPDESC_PROPERTIES;
        default : return null;
      }
    }
    return null;
  }

  /**
   * @see ro.sync.ecss.extensions.commons.table.operations.InsertRowOperationBase#getRowElementName(AuthorElement)
   */
  @Override
  protected String getRowElementName(AuthorElement tableElement) {
    String tableName = tableElement.getLocalName();
    if (ELEMENT_NAME_SIMPLETABLE.equals(tableName)) {
      // Simpletable
      return ELEMENT_NAME_ROW_SIMPLETABLE;
    } else if (ELEMENT_NAME_CHOICETABLE.equals(tableName)) {
      // Choicetable
      return ELEMENT_NAME_ROW_CHOICETABLE;
    } else if (ELEMENT_NAME_PROPERTIES.equals(tableName)) {
      // Proporties
      return ELEMENT_NAME_ROW_PROPERTIES;
    }
    return null;
  }
  
  /**
   * @see ro.sync.ecss.extensions.commons.table.operations.InsertRowOperationBase#getDefaultContentForEmptyCells()
   */
  @Override
  protected String getDefaultContentForEmptyCells() {
    return cellContent;
  }
  
  /**
   * @see ro.sync.ecss.extensions.commons.table.operations.InsertRowOperationBase#getOperationArguments()
   */
  @Override
  protected ArgumentDescriptor[] getOperationArguments() {
    ArgumentDescriptor[] arguments = null;
    ArgumentDescriptor[] args = super.getOperationArguments();
    if (args != null) {
      arguments = new ArgumentDescriptor[args.length + 1];
      // Add descriptors from super
      for (int i = 0; i < args.length; i++) {
        arguments[i] = args[i];
      }
      // Add cell fragment argument
      arguments[args.length] = CELL_FRAGMENT_ARGUMENT;
    } else {
      arguments = CELL_FRAGMENT_ARGUMENT_IN_ARRAY;
    }
    return arguments;
  }
}