/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.exml.plugin.option;

import javax.swing.JComponent;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.OptionsStorage;
import ro.sync.exml.plugin.PluginExtension;
import ro.sync.exml.workspace.api.PluginWorkspace;

/**
 * Class used to create plugin option page extension. It receives callbacks for
 * saving options, restoring default options and loading options.
 * The GUI for this option page must be built in order to associated the options
 * with their corresponding GUI components. 
 * 
 * @since 15
 * 
 * <br>
 * ********************************
 * <br>
 * EXPERIMENTAL - Subject to change
 * <br>
 * *********************************
 * </br>
 * <p>Please note that this API is not marked as final and it can change in one of the next 
 * versions of the application. If you have suggestions, comments about it, please let us know.</p> 
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public abstract class OptionPagePluginExtension implements PluginExtension {

  /**
   * This method is called when "Apply" or "OK" button are pressed in from the GUI 
   * option page. All options associated with the option page must be saved on this method. 
   * 
   * @param pluginWorkspace Access the entire workspace of Oxygen. It can be used to
   * retrieve the {@link OptionsStorage} and perform options save operations on it.
   */
  public abstract void apply(PluginWorkspace pluginWorkspace);
  
  /**
   * This method is called when "Restore defaults" button is pressed from the GUI
   * option page. All options associated with the option page must be restored to their default values.
   */
  public abstract void restoreDefaults();
  
  /**
   * Retrieves the option page title.
   * 
   * @return The option page title used in GUI.
   */
  public abstract String getTitle();
  
  /**
   * Initializes the GUI for the option page and loads the stored option values.
   * 
   * @param pluginWorkspace Access the entire workspace of Oxygen. It can be used to
   * retrieve the {@link OptionsStorage} and perform options save/load operations it.
   * 
   * @return The GUI component of the option page.
   */
  public abstract JComponent init(PluginWorkspace pluginWorkspace);
}
