/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.exml.plugin.urlstreamhandler;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;

import java.net.URL;

/**
 * This interface can be called to decide if an URL is read-only or not.
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public interface URLHandlerReadOnlyCheckerExtension {
  /**
   * Decide if a given URL is read-only or writable to. 
   * If an URL is marked as read-only Oxygen will present it with a read-only lock icon on the tab.
   * The user will also be able to specify through the user interface that an opened editor over a read-only URL can be edited.
   * Also when saving in Oxygen and the URL is marked as read-only the Save As dialog will be shown instead.
   *   
   * 
   * This call back will usually be received often, each time focus is gained by the current editor.
   * WARNING: It is advisable to cache the calls for recognized URLs as this method might be called quite often.  
   * 
   * @param url The URL for which the implementor will decide if it is read-only or not.
   * 
   * @return true if a given URL should be regarded by Oxygen as read-only.
   */
  public boolean isReadOnly(URL url);
  
  /**
   * Returns true if the implementor should be asked about the read-only status for URL's of a certain protocol.
   * 
   * This call back will usually be received often, each time focus is gained by the current editor.
   * 
   * @param protocol The URL protocol (like "http" or "file")
   * @return <code>true</code> if the implementor should be asked about the read-only status for URL's of a certain protocol.
   * 
   * For example if you want to decide if "http" URLs are read-only or not you should return <code>true</code> if the protocol is "http"
   * Then, the other callback will be called and you will be able to decide the read-only status for the URL.
   */
  public boolean canCheckReadOnly(String protocol);
}