/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.licensemanager;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;

/**
 * All components for an oXygen license.
 * 
 * Usually a license has several components licensed like:<br>
 * {@link #EDITOR_COMPONENT_NAME} and {@link #SAXON_SA_COMPONENT_NAME}
 * for XML Editor
 * <br>
 * or
 * <br>
 * {@link #AUTHOR_COMPONENT_NAME} and {@link #SAXON_SA_COMPONENT_NAME}
 * for XML Author
 *  
 * @since 12.2
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public interface LicenseComponents {
  /**
   * The Author component is licensed both on SA and Eclipse.
   */
  public static final String AUTHOR_COMPONENT_NAME = "Author";
  /**
   * The AuthorComponent component is licensed for using it 
   * as a component of another application or applet.
   */
  public static final String AUTHOR_COMPONENT_COMPONENT_NAME = "Author-Component";  
  /**
   * Webapp component license.
   */
  public static final String WEBAPP_COMPONENT_NAME = "Webapp";  
  /**
   * The Editor component is licensed both on SA and Eclipse.
   */
  public static final String EDITOR_COMPONENT_NAME = "XML-Editor";
  /**
   * The Developer component is licensed both on SA and Eclipse.
   */
  public static final String DEVELOPER_COMPONENT_NAME = "XML-Developer";
  /**
   * The Debugger component is licensed both on SA and Eclipse.
   */
  public static final String DEBUGGER_COMPONENT_NAME = "XSLT-Debugger";
  /**
   * The SVN Client component name.
   */
  public static final String SVN_CLIENT_COMPONENT_NAME = "SVN-Client";
  /**
   * The Saxon SA component.
   */
  public static final String SAXON_SA_COMPONENT_NAME = "Saxon-SA";
  /**
   * The Diff component name for the server.
   */
  public static final String DIFF_SERVER_COMPONENT_NAME = "Diff-Srv";
  /**
   * The Diff application component name.
   */
  public static final String DIFF_APPLICATION_COMPONENT_NAME = "Diff-App";
  /**
   * The component name for using Oxygen tools from command line. For example:
   * 
   * <ul>
   *   <li>Webhelp-plugin used in external DITA-OT, see EXM-27527 for more details </li>
   *   <li>XSD documentation generator called from command line </li>
   *   <li>XSLT documentation generator called from command line </li>
   * </ul>
   */
  public static final String OXYGEN_SCRIPTING_COMPONENT_NAME = "Scripting";
}